@@ -53,8 +53,11 @@ autosummary_generate = True
 autodoc_typehints = 'description'
 autodoc_type_aliases = dict(
     ArrayLike='ArrayLike',
+    DType='DType',
     DTypeLike='DTypeLike',
     Indexer='Indexer',
+    NDArray='NDArray',
+    NDMaskedArray='NDMaskedArray',
     PartitionCallback='PartitionCallback',
     QueryDict='QueryDict',
     Scalar='Scalar',
@@ -95,7 +98,7 @@ intersphinx_mapping = {
 
 # -- Extension configuration -------------------------------------------------
 sphinx_gallery_conf = {
-    'examples_dirs': [HERE.joinpath('examples')],
+    'examples_dirs': [HERE.parent.parent.joinpath('examples')],
     'filename_pattern': r'[\\\/]ex_',
     'pypandoc': False,
 }
