@@ -85,6 +85,7 @@ def variables(
     """Return the variables defined in the dataset.
 
     Args:
+        metadata: Metadata dataset containing variables information.
         selected_variables: The variables to return. If None, all the
             variables are returned.
 
@@ -157,8 +158,13 @@ def _immutable_path(
             (partition_properties.dim, ), predicate=False) else None
 
 
-def _write_immutable_dataset(ds: dataset.Dataset, axis: str, path: str,
-                             fs: fsspec.AbstractFileSystem) -> None:
+def _write_immutable_dataset(
+    ds: dataset.Dataset,
+    axis: str,
+    path: str,
+    fs: fsspec.AbstractFileSystem,
+    synchronizer: sync.Sync,
+) -> None:
     """Write the immutable dataset.
 
     Args:
@@ -166,7 +172,6 @@ def _write_immutable_dataset(ds: dataset.Dataset, axis: str, path: str,
         axis: The partitioning axis.
         path: The path to the immutable dataset.
         fs: The file system that the partition is stored on.
-        synchronizer: The synchronizer to use.
     """
     immutable_dataset = ds.select_variables_by_dims((axis, ), predicate=False)
     assert len(immutable_dataset.variables) != 0, (
@@ -176,7 +181,7 @@ def _write_immutable_dataset(ds: dataset.Dataset, axis: str, path: str,
     storage.write_zarr_group(immutable_dataset,
                              path,
                              fs,
-                             sync.NoSync(),
+                             synchronizer,
                              parallel=False)
 
 
@@ -329,6 +334,10 @@ class Collection:
         """Return True if the collection is read-only."""
         return self.mode == 'r'
 
+    def is_locked(self) -> bool:
+        """Return True if the collection is locked."""
+        return self.synchronizer.is_locked()
+
     @classmethod
     def from_config(
         cls,
@@ -394,6 +403,7 @@ class Collection:
         *,
         merge_callable: merging.MergeCallable | None = None,
         npartitions: int | None = None,
+        validate: bool = False,
     ) -> None:
         """Insert a dataset into the collection.
 
@@ -404,8 +414,9 @@ class Collection:
                 None, the new partitioned data overwrites the existing
                 partitioned data.
             npartitions: The maximum number of partitions to process in
-                parallel. By default, all partitions are processed in parallel
-                in a single task.
+                parallel. By default, partitions are processed one by one.
+            validate: Whether to validate dataset metadata before insertion
+                or not.
 
         .. warning::
 
@@ -431,16 +442,21 @@ class Collection:
             variable = self.metadata.variables[item]
             ds.add_variable(variable)
 
+        if validate and not ds.metadata() == self.metadata:
+            raise ValueError(
+                "Provided dataset's metadata do not match the collection's ones"
+            )
+        ds = ds.set_for_insertion(ds=self.metadata)
         client = dask_utils.get_client()
 
         # If the dataset contains variables that should not be partitioned.
         if self._immutable is not None:
 
-            # On the first call, we store the immutable variables in a
+            # On the first call, we store the immutable variables in
             # a directory located at the root of the collection.
             if not self.fs.exists(self._immutable):
                 _write_immutable_dataset(ds, self.axis, self._immutable,
-                                         self.fs)
+                                         self.fs, self.synchronizer)
 
             # Remove the variables that should not be partitioned.
             ds = ds.select_variables_by_dims((self.axis, ))
@@ -455,19 +471,22 @@ class Collection:
                 raise ValueError('The number of partitions must be positive')
             npartitions = len(partitions) // npartitions + 1
 
-        scattared_ds = client.scatter(ds)
+        scattered_ds = client.scatter(ds)
         for sequence in dask_utils.split_sequence(partitions, npartitions):
             futures = [
                 dask_utils.simple_delayed('insert', _insert)(
                     partition,  # type: ignore[arg-type]
                     axis=self.axis,
-                    ds=scattared_ds,
+                    ds=scattered_ds,
                     fs=self.fs,
                     merge_callable=merge_callable,
                     partitioning_properties=self.partition_properties,
                 ) for partition in sequence
             ]
-            client.compute(futures, sync=True)
+            storage.execute_transaction(client,
+                                        self.synchronizer,
+                                        futures,
+                                        sync=True)
 
     def _relative_path(self, path: str) -> str:
         """Return the relative path to the collection.
@@ -484,12 +503,15 @@ class Collection:
     def partitions(
         self,
         *,
+        lock: bool = False,
         filters: PartitionFilter = None,
         relative: bool = False,
     ) -> Iterator[str]:
         """List the partitions of the collection.
 
         Args:
+            lock: Whether to lock the collection or not to avoid listing
+                partitions while the collection is being modified.
             filters: The predicate used to filter the partitions to load. If
                 the predicate is a string, it is a valid python expression to
                 filter the partitions, using the partitioning scheme as
@@ -518,7 +540,14 @@ class Collection:
         base_dir = self.partition_properties.dir
         sep = self.fs.sep
 
-        for item in self.partitioning.list_partitions(self.fs, base_dir):
+        if lock:
+            with self.synchronizer:
+                partitions: Iterable[str] = tuple(
+                    self.partitioning.list_partitions(self.fs, base_dir))
+        else:
+            partitions = self.partitioning.list_partitions(self.fs, base_dir)
+
+        for item in partitions:
             if item == self._immutable:
                 continue
             # Filtering on partition names
@@ -552,7 +581,7 @@ class Collection:
         """
         now = datetime.datetime.now()
         client = dask_utils.get_client()
-        folders = list(self.partitions(filters=filters))
+        folders = list(self.partitions(filters=filters, lock=True))
 
         def is_created_before(path: str) -> bool:
             created = self.fs.created(path)
@@ -671,7 +700,7 @@ class Collection:
                 To get more information on the predicate, see the
                 documentation of the :meth:`partitions` method.
             partition_size: The length of each bag partition.
-            npartitions: The number of desired bag partitions.
+            npartition: The number of desired bag partitions.
             selected_variables: A list of variables to retain from the
                 collection. If None, all variables are kept.
             **kwargs: The keyword arguments to pass to the function.
@@ -773,7 +802,7 @@ class Collection:
             ...     keys["month"] == 3 and keys["day"] % 2 == 0)
         """
         client = dask_utils.get_client()
-        arrays: list[dataset.Dataset] = []
+        arrays: list[dataset.Dataset]
         if indexer is None:
             selected_partitions = tuple(self.partitions(filters=filters))
             if len(selected_partitions) == 0:
@@ -815,6 +844,7 @@ class Collection:
             array.merge(
                 storage.open_zarr_group(self._immutable, self.fs,
                                         selected_variables))
+        array.fill_attrs(ds=self.metadata)
         return array
 
     # pylint: disable=method-hidden
@@ -844,7 +874,7 @@ class Collection:
                 dataset the selected partition.
             filters: The expression used to filter the partitions to update.
             partition_size: The number of partitions to update in a single
-                batch. By default 1, which is the same as to map the function to
+                batch. By default, 1 which is the same as to map the function to
                 each partition. Otherwise, the function is called on a batch of
                 partitions.
             selected_variables: A list of variables to load from the collection.
@@ -866,11 +896,12 @@ class Collection:
             one_partition = next(self.partitions(filters=filters))
         except StopIteration:
             return
-        func_result = try_infer_callable(
-            func,
-            storage.open_zarr_group(one_partition, self.fs,
-                                    selected_variables),
-            self.partition_properties.dim, *args, **kwargs)
+        with self.synchronizer:
+            ds = storage.open_zarr_group(one_partition, self.fs,
+                                         selected_variables)
+        func_result = try_infer_callable(func, ds,
+                                         self.partition_properties.dim, *args,
+                                         **kwargs)
         unknown_variables = set(func_result) - set(
             self.metadata.variables.keys())
         if len(unknown_variables):
@@ -887,25 +918,30 @@ class Collection:
         _LOGGER.info('Updating of the (%s) variable in the collection',
                      ', '.join(repr(item) for item in func_result))
         client = dask_utils.get_client()
-        batchs = dask_utils.split_sequence(
-            tuple(self.partitions(filters=filters)), partition_size
+
+        batches = dask_utils.split_sequence(
+            tuple(self.partitions(filters=filters, lock=True)), partition_size
             or dask_utils.dask_workers(client, cores_only=True))
-        awaitables = client.map(local_func, tuple(batchs), key=func.__name__)
-        storage.execute_transaction(client, self.synchronizer, awaitables)
+        storage.execute_transaction(
+            client, self.synchronizer,
+            client.map(local_func, tuple(batches), key=func.__name__))
 
     def _bag_from_partitions(
         self,
         filters: PartitionFilter | None = None,
+        **kwargs,
     ) -> dask.bag.core.Bag:
         """Return a dask bag from the partitions.
 
         Args:
             filters: The predicate used to filter the partitions to load.
+            kwargs: The keyword arguments to pass to the method
+                :meth:`partitions`.
 
         Returns:
             The dask bag.
         """
-        partitions = [*self.partitions(filters=filters)]
+        partitions = [*self.partitions(filters=filters, **kwargs)]
         return dask.bag.core.from_sequence(seq=partitions,
                                            npartitions=len(partitions))
 
@@ -941,8 +977,11 @@ class Collection:
                 raise ValueError(
                     f'The variable {variable!r} is part of the immutable '
                     'dataset.')
-        bag = self._bag_from_partitions()
-        bag.map(storage.del_zarr_array, variable, self.fs).compute()
+        client = dask_utils.get_client()
+        bag = self._bag_from_partitions(lock=True)
+        awaitables = dask.distributed.futures_of(
+            bag.map(storage.del_zarr_array, variable, self.fs).persist())
+        storage.execute_transaction(client, self.synchronizer, awaitables)
         del self.metadata.variables[variable]
         self._write_config()
 
@@ -982,11 +1021,16 @@ class Collection:
         self.metadata.add_variable(variable)
         self._write_config()
 
+        client = dask_utils.get_client()
+
         template = self.metadata.search_same_dimensions_as(variable)
+        chunks = {dim.name: dim.value for dim in self.metadata.chunks}
         try:
-            bag = self._bag_from_partitions()
-            bag.map(storage.add_zarr_array, variable, template.name,
-                    self.fs).compute()
+            bag = self._bag_from_partitions(lock=True)
+            futures = dask.distributed.futures_of(
+                bag.map(storage.add_zarr_array, variable, template.name,
+                        self.fs, chunks).persist())
+            storage.execute_transaction(client, self.synchronizer, futures)
         except Exception:
             self.drop_variable(variable.name)
             raise
