@@ -8,10 +8,12 @@ Test of the collections
 """
 from __future__ import annotations
 
+import concurrent.futures
 import datetime
 import io
 
 import dask.array.core
+import dask.distributed
 import fsspec
 import numpy
 import pytest
@@ -25,6 +27,8 @@ from ... import (
     meta,
     partitioning,
     storage,
+    sync,
+    variable,
 )
 # pylint: disable=unused-import # Need to import for fixtures
 from ...tests.cluster import dask_client, dask_cluster
@@ -97,6 +101,7 @@ def test_insert(
     dask_client,  # pylint: disable=redefined-outer-name,unused-argument
     arg,
     request,
+    tmpdir,
 ):
     """Test the insertion of a dataset."""
     tested_fs = request.getfixturevalue(arg)
@@ -106,7 +111,9 @@ def test_insert(
                                         ds.metadata(),
                                         partitioning.Date(('time', ), 'D'),
                                         str(tested_fs.collection),
-                                        filesystem=tested_fs.fs)
+                                        filesystem=tested_fs.fs,
+                                        synchronizer=sync.ProcessSync(
+                                            str(tmpdir / 'lock.lck')))
 
     indices = numpy.arange(0, len(datasets))
     numpy.random.shuffle(indices)
@@ -513,10 +520,7 @@ def test_insert_with_missing_variable(
             ds.variables['var1'].fill_value))
 
 
-# For the moment, this test does not work with S3: minio creates a
-# directory for the file "time"; therefore zarr cannot detect an invalid
-# array.
-@pytest.mark.parametrize('arg', ['local_fs'])  # , "s3_fs"])
+@pytest.mark.parametrize('arg', ['local_fs', 's3_fs'])
 def test_insert_failed(
     dask_client,  # pylint: disable=redefined-outer-name,unused-argument
     arg,
@@ -531,17 +535,64 @@ def test_insert_failed(
                                         str(tested_fs.collection),
                                         filesystem=tested_fs.fs)
     partitions = list(zcollection.partitioning.split_dataset(ds, 'time'))
-    one_directory = zcollection.fs.sep.join(
-        (zcollection.partition_properties.dir, ) + partitions[0][0])
-    zcollection.fs.makedirs(one_directory, exist_ok=False)
-    zcollection.fs.touch(zcollection.fs.sep.join((one_directory, 'time')))
 
-    with pytest.raises((KeyError, OSError)):
+    # Create a file in the directory where the dataset should be written. This
+    # should cause the insertion to fail.
+    sep = zcollection.fs.sep
+    one_directory = sep.join((zcollection.partition_properties.dir, ) +
+                             partitions[0][0])
+    zcollection.fs.makedirs(sep.join(one_directory.split(sep)[:-1]))
+    zcollection.fs.touch(one_directory)
+
+    with pytest.raises((OSError, ValueError)):
         zcollection.insert(ds)
 
+    # Because the insert failed, the partition that was supposed to be created
+    # was deleted.
+    assert not zcollection.fs.exists(one_directory)
     zcollection.insert(ds)
 
 
+@pytest.mark.parametrize('arg', ['local_fs', 's3_fs'])
+def test_insert_validation(
+    dask_client,  # pylint: disable=redefined-outer-name,unused-argument
+    arg,
+    request,
+):
+    """Test the insertion of a dataset with metadata validation."""
+    tested_fs = request.getfixturevalue(arg)
+    ds = next(create_test_dataset_with_fillvalue())
+
+    zcollection = convenience.create_collection(
+        axis='time',
+        ds=ds,
+        partition_handler=partitioning.Date(('time', ), 'M'),
+        partition_base_dir=str(tested_fs.collection),
+        filesystem=tested_fs.fs)
+    zcollection.insert(ds, merge_callable=merging.merge_time_series)
+
+    ds = next(create_test_dataset_with_fillvalue())
+
+    # Inserting a dataset containing valid attributes
+    zcollection.insert(ds, validate=True)
+
+    # Inserting a dataset containing an invalid attributes
+    ds = next(create_test_dataset_with_fillvalue())
+    ds.attrs = (meta.Attribute('invalid', 1), )
+
+    with pytest.raises(ValueError):
+        zcollection.insert(ds, validate=True)
+
+    # Inserting a dataset containing variables with invalid attributes
+    ds = next(create_test_dataset_with_fillvalue())
+
+    for var in ds.variables.values():
+        var.attrs = (meta.Attribute('invalid', 1), )
+
+    with pytest.raises(ValueError):
+        zcollection.insert(ds, validate=True)
+
+
 @pytest.mark.parametrize('arg', ['local_fs', 's3_fs'])
 def test_map_partition(
     dask_client,  # pylint: disable=redefined-outer-name,unused-argument
@@ -760,3 +811,111 @@ def test_copy_collection(
                      ds_after_copy.variables['var1'].values)
     assert numpy.all(ds_before_copy.variables['var2'].values ==
                      ds_after_copy.variables['var2'].values)
+
+
+def _insert(ds: dataset.Dataset, base_dir: str, lock_file: str,
+            scheduler_file: str):
+    client = dask.distributed.Client(scheduler_file=scheduler_file)
+    zcollection = collection.Collection.from_config(
+        base_dir, mode='w', synchronizer=sync.ProcessSync(lock_file))
+    zcollection.insert(ds, merge_callable=merging.merge_time_series)
+
+
+# pylint: disable=too-many-statements
+def test_concurrent_insert(
+    dask_client,  # pylint: disable=redefined-outer-name,unused-argument
+    tmpdir,
+):
+    """Test the insertion of a dataset."""
+    fs = fsspec.filesystem('file')
+    datasets = list(create_test_dataset())
+    ds = datasets[0]
+    lock_file = str(tmpdir / 'lock.lck')
+    synchronizer = sync.ProcessSync(lock_file)
+    base_dir = str(tmpdir / 'test')
+    zcollection = collection.Collection('time',
+                                        ds.metadata(),
+                                        partitioning.Date(('time', ), 'D'),
+                                        base_dir,
+                                        filesystem=fs,
+                                        synchronizer=synchronizer)
+
+    pool = concurrent.futures.ProcessPoolExecutor(max_workers=8)
+    futures = []
+
+    assert zcollection.is_locked() is False
+
+    indices = numpy.arange(0, len(datasets))
+    numpy.random.shuffle(indices)
+    futures = [
+        pool.submit(_insert, datasets[ix], base_dir, lock_file,
+                    dask_client.scheduler_file) for ix in indices
+    ]
+
+    def update(ds: dataset.Dataset, shift: int = 3):
+        """Update function used for this test."""
+        return dict(var2=ds.variables['var1'].values * -1 + shift)
+
+    launch_update = True
+    for item in concurrent.futures.as_completed(futures):
+        if launch_update:
+            zcollection.update(update)  # type: ignore
+            launch_update = False
+
+    data = zcollection.load()
+    assert data is not None
+    values = data.variables['time'].values
+    assert numpy.all(values == numpy.arange(START_DATE, END_DATE, DELTA))
+
+
+def test_partition_modified(
+        dask_client,  # pylint: disable=redefined-outer-name,unused-argument
+        tmpdir):
+    """Test the loading of a variable that has been modified since its
+    creation."""
+    fs = fsspec.filesystem('file')
+    datasets = list(create_test_dataset())
+    ds = datasets[0]
+    base_dir = str(tmpdir / 'test')
+    zcollection = collection.Collection('time',
+                                        ds.metadata(),
+                                        partitioning.Date(('time', ), 'D'),
+                                        base_dir,
+                                        filesystem=fs)
+    zcollection.insert(ds)
+
+    last_month = zcollection.load(filters=lambda keys: keys['year'] == 2000 and
+                                  keys['month'] == 1 and keys['day'] == 16)
+    assert last_month is not None
+
+    ds = zcollection.load()
+    assert ds is not None
+
+    def new_shape(
+        var: variable.Variable,
+        selected_dim: str,
+        new_size: int,
+    ) -> tuple[int, ...]:
+        """Compute the new shape of a variable."""
+        return tuple(new_size if dim == selected_dim else size
+                     for dim, size in zip(var.dimensions, var.shape))
+
+    dim, size = 'num_lines', last_month.dimensions['num_lines'] * 25
+    last_month = dataset.Dataset([
+        variable.Variable(
+            name,
+            numpy.resize(var.array.compute(), new_shape(var, dim, size)),
+            var.dimensions,
+            var.attrs,
+            var.compressor,
+            var.fill_value,
+            var.filters,
+        ) for name, var in last_month.variables.items()
+    ])
+    zcollection.insert(last_month)
+
+    with pytest.raises(RuntimeError, match='Try to re-load'):
+        _ = ds['var2'].values
+
+    with pytest.raises(RuntimeError, match='Try to re-load'):
+        _ = ds['time'].values
