@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -10,7 +10,7 @@ from __future__ import annotations
 
 import xarray
 
-from .. import collection, dataset, partitioning, utilities
+from .. import collection, dataset, fs_utils, partitioning
 
 
 def create_collection(
@@ -48,7 +48,7 @@ def create_collection(
     Raises:
         ValueError: If the base directory already exists.
     """
-    filesystem = utilities.get_fs(kwargs.pop('filesystem', None))
+    filesystem = fs_utils.get_fs(kwargs.pop('filesystem', None))
     if filesystem.exists(partition_base_dir):
         raise ValueError(
             f'The directory {partition_base_dir!r} already exists.')
