@@ -10,6 +10,7 @@ from __future__ import annotations
 
 from typing import Protocol
 import random
+import shutil
 
 import fsspec
 import zarr.storage
@@ -59,6 +60,31 @@ class MergeCallable(Protocol):
     #: pylint: enable=too-few-public-methods,duplicate-code
 
 
+def _rename(
+    fs: fsspec.AbstractFileSystem,
+    source: str,
+    dest: str,
+):
+    """Rename a directory on a file system.
+
+    Args:
+        fs: The file system.
+        source: The source directory.
+        dest: The destination directory.
+    """
+    if isinstance(fs, fsspec.implementations.local.LocalFileSystem):
+        # fspec implementation of the local file system, copy the source
+        # directory to the destination directory and remove the source
+        # directory. This is not efficient. So we use the shutil
+        # implementation to rename the directory.
+        shutil.rmtree(dest, ignore_errors=True)
+        shutil.move(source, dest)
+        return
+
+    fs.rm(dest, recursive=True)
+    fs.mv(source, dest, recursive=True)
+
+
 def _update_fs(
     dirname: str,
     ds: dataset.Dataset,
@@ -91,7 +117,7 @@ def _update_fs(
         raise
 
     # Rename the existing entry on the file system
-    fs.rename(temp, dirname, recursive=True)
+    _rename(fs, temp, dirname)
 
 
 def perform(
