@@ -22,13 +22,10 @@ import fsspec
 import numpy
 import zarr
 
-from . import dask_utils, dataset, meta, sync
+from . import dataset, meta, sync
 from .fs_utils import join_path
 from .type_hints import ArrayLike
 
-#: Block size limit used with dask arrays. (128 MiB)
-BLOCK_SIZE_LIMIT = 134217728
-
 #: Name of the attribute storing the names of the dimensions of an array.
 DIMENSIONS = '_ARRAY_DIMENSIONS'
 
@@ -46,6 +43,7 @@ def execute_transaction(
     client: dask.distributed.Client,
     synchronizer: sync.Sync,
     futures: Sequence[dask.distributed.Future | dask_Delayed],
+    **kwargs: Any,
 ) -> Any:
     """Execute a transaction in the collection.
 
@@ -53,6 +51,7 @@ def execute_transaction(
         client: The Dask client.
         synchronizer: The instance handling access to critical resources.
         futures: Lazy tasks to be done.
+        kwargs: Keyword arguments to pass to :func:`dask.distributed.compute`.
 
     Returns:
         The result of the transaction.
@@ -62,7 +61,7 @@ def execute_transaction(
     awaitables = []
     try:
         with synchronizer:
-            awaitables = client.compute(futures)
+            awaitables = client.compute(futures, **kwargs)
             return client.gather(awaitables)
     except:  # noqa: E722
         # Before throwing the exception, we wait until all future scheduled
@@ -125,6 +124,8 @@ def write_zarr_variable(
     args: tuple[str, dataset.Variable],
     dirname: str,
     fs: fsspec.AbstractFileSystem,
+    chunks: dict[str, int | str] | None = None,
+    block_size_limit: int | None = None,
 ) -> None:
     """Write a variable to a Zarr dataset.
 
@@ -134,15 +135,26 @@ def write_zarr_variable(
             - The variable to write.
         dirname: The target directory.
         fs: The file system on which the Zarr dataset is stored.
+        chunks: Chunk size for each dimension.
+        block_size_limit: Maximum size (in bytes) of a block/chunk.
     """
     name, variable = args
     kwargs = {'filters': variable.filters}
     data = variable.array
 
-    chunks = {ix: -1 for ix in range(variable.ndim)}
+    # If the user has not specified a chunk size, we use the default one.
+    # Otherwise, we use the user's choice.
+    block_size_limit = block_size_limit or meta.BLOCK_SIZE_LIMIT
+    var_chunks: dict[int, int | str] = {
+        ix: -1
+        for ix in range(variable.ndim)
+    } if chunks is None else {
+        ix: chunks.get(dim, -1)
+        for ix, dim in enumerate(variable.dimensions)
+    }
     data = data.rechunk(
-        chunks,  # type: ignore[arg-type]
-        block_size_limit=BLOCK_SIZE_LIMIT,
+        var_chunks,  # type: ignore[arg-type]
+        block_size_limit=block_size_limit,
     )
 
     _to_zarr(array=data,
@@ -196,16 +208,24 @@ def write_zarr_group(
         synchronizer: The instance handling access to critical resources.
         parallel: Whether to write the variables in parallel using Dask.
     """
-    futures = [
-        dask_utils.simple_delayed(name, write_zarr_variable)((name, variable),
-                                                             dirname, fs)
-        for name, variable in ds.variables.items()
-    ]
     if parallel:
         with dask.distributed.worker_client() as client:
+            iterables = [(name, client.scatter(variable))
+                         for name, variable in ds.variables.items()]
+            futures = client.map(write_zarr_variable,
+                                 iterables,
+                                 dirname=dirname,
+                                 fs=fs,
+                                 chunks=ds.chunks,
+                                 block_size_limit=ds.block_size_limit)
             execute_transaction(client, synchronizer, futures)
     else:
-        dask.base.compute(*futures, scheduler=dask.local.get_sync)
+        for name, variable in ds.variables.items():
+            write_zarr_variable((name, variable),
+                                dirname,
+                                fs,
+                                chunks=ds.chunks,
+                                block_size_limit=ds.block_size_limit)
     _write_meta(ds, dirname, fs)
 
 
@@ -239,7 +259,7 @@ def open_zarr_group(
     """
     _LOGGER.debug('Opening Zarr group %r', dirname)
     store: zarr.Group = zarr.open_consolidated(  # type: ignore[arg-type]
-        fs.get_mapper(dirname), )
+        fs.get_mapper(dirname), mode='r')
     # Ignore unknown variables to retain.
     selected_variables = set(selected_variables) & set(
         store) if selected_variables is not None else set(store)
@@ -309,6 +329,7 @@ def add_zarr_array(
     variable: meta.Variable,
     template: str,
     fs: fsspec.AbstractFileSystem,
+    chunks: dict[str, int | str] | None = None,
 ) -> None:
     """Add a variable to a Zarr dataset.
 
@@ -317,14 +338,20 @@ def add_zarr_array(
         variable: The variable to add.
         template: The name of the template variable.
         fs: The file system that the dataset is stored on.
+        chunks: Chunk size for each dimension.
     """
     _LOGGER.debug('Adding variable %r to Zarr dataset %r', variable.name,
                   dirname)
     shape = zarr.open(fs.get_mapper(join_path(dirname, template))).shape
+
+    var_chunks = shape if chunks is None else tuple(
+        chunks.get(dim, shape[ix])
+        for ix, dim in enumerate(variable.dimensions))
+
     store = fs.get_mapper(join_path(dirname, variable.name))
     zarr.create(
         shape,
-        chunks=True,
+        chunks=var_chunks,
         dtype=variable.dtype,
         compressor=variable.compressor,  # type: ignore[arg-type]
         fill_value=variable.fill_value,  # type: ignore[arg-type]
