@@ -1,4 +1,4 @@
-# Copyright (c) 2022 CNES
+# Copyright (c) 2023 CNES
 #
 # All rights reserved. Use of this source code is governed by a
 # BSD-style license that can be found in the LICENSE file.
@@ -43,7 +43,8 @@ def dask_cluster(pytestconfig, tmpdir_factory, scope='session'):
         protocol='tcp://',
         n_workers=n_workers,
         threads_per_worker=threads_per_worker,
-        processes=processes)
+        processes=processes,
+    )
 
     def teardown():
         """Stop the cluster and remove the scheduler file."""
